function [t,v,vin]=H2hpulse(f,H,Tr,UI,Tend,Td);
%
%   Create time domain step responses from S-parameters
%
%   Input
%           f=frequency
%           S=S-matrix
%           Tr= rise time
%           Tend=end of the time duration
%           Td=delay of the input step
%           UI=pulse width

n=length(f);

% Check if the sampling is uniform
    df=f(3)-f(2); df1=f(4)-f(3);
    
    if abs(df-df1) > 1 
        error('Frequency sampling is not uniform...');
    end
    
% Check if DC point is available
%    if f(1) > 1
%        warning('No DC point......');
%    end
    
% Force the input duration Tend <= max time Tt  
    dt=1/(n*df);
    %dt=0.5/(n*df); 
    Tt=(n-1)*dt;
    if Tend > Tt; 
        Tend=Tt; 
    end
    
% Input waveform
    f1=f; if f1(1)==0; f1(1)=1e-30; end;
    s=complex(0,2*pi*f1);
    V=1./(Tr*s.^2).*(1.-exp(-s*Tr)).*(1.-exp(-s*UI)).*exp(-s*Td);
    vin=2*real(ifft(V)/dt); vin=vin-vin(1);
           
    
% ifft
    t=[0:dt:Tend]';
    nt=length(t);
    vout=[];
    F=V.*H;
    vout=2*real(ifft(F)/dt);
    vout=vout-vout(1);         % force waveform to start from zero
    v=vout(1:nt);
    tmp=vin(1:nt); vin=[]; vin=tmp;    
%
% End of the function
%