%  Process low speed data   

columnname = {'Value', 'Margin/Criteria', 'Pass/Fail'};        
rowname = {'CC to D+/D-','CC to D- (CT)','VBUS to D+/D-',...
           'VBUS Lloop','M(VBUS,D+)/M(VBUS,D-)',...
           'M(VBUS,CC)'}; 
set(handles.uitable_result,'ColumnName', columnname);
set(handles.uitable_result,'RowName', rowname);
set(handles.uitable_result,'Data', []);

% Ref impedance for LS is always 50 ohms
Zref=inp.Z0_low;

% Number of ports and number of S-matrices
if VNA12, m=12; n=1; 
elseif VNA8, m=8; n=1;
elseif VNA4 m=4; n=6; end

% Renorm
for i=1:n
    
    % Get ith S-matrix
    f=rawSp{i}.f;
    S=rawSp{i}.S;
    Z0=rawSp{i}.Z0;    
    
    % Renoramlize impedance
    if Zref ~= Z0
        rho=(Zref-Z0)/(Zref+Z0);
        for k=1:size(S1,3)
            new{i}.S(:,:,k)=inv(eye(m,m)-rho*S(:,:,k))*(S(:,:,k)-rho*eye(m,m));
        end
    else
        new{i}.S=S;
    end
    
end

        % Populate ploting popupmenu
        itemStr{1}='CC to USB D+/D-';
        itemStr{2}='CC to USB D-';
        itemStr{3}='VBUS to USB D+/D-';           
        set(handles.popupmenu_Spec,'String',itemStr);
        set(handles.popupmenu_Spec,'Value',1);
    
% Assemble the LS S-parameters matrix to 12 ports
%-------------------------------------------------------------------------
N=size(new{1}.S,3);
S=zeros(8,8,N);

if VNA12
    S=new{1}.S(1:8,1:8,:);
elseif VNA8
    S=new{1}.S; 
elseif VNA4
    portMap=[1 2 3 4;...
             1 2 5 6;...
             1 2 7 8;...             
             3 4 5 6;...
             3 4 7 8;...             
             5 6 7 8];
    for i=1:n
        S(portMap(i,:),portMap(i,:),:)=new{i}.S;
    end     
end

% Frequency range to extract inductances
flower=inp.flower_low; 
fupper=inp.fupper_low;    
    
% Extract Vbus loop inductance
%--------------------------------------------------------------------------
ID= find(f>=flower & f<=fupper);
f_vbus=f(ID);
if ~isempty(f_vbus)
    S55=squeeze(S(5,5,ID));
    S56=squeeze(S(5,6,ID));
    S65=squeeze(S(6,5,ID));
    S66=squeeze(S(6,6,ID));
else
    disp('Frequency of VBUS measurment is out of range');
    S56=[]; S65=[];S55=[]; S66=[];
end     

omega=2*pi*f_vbus;

% Use Apple's formula
Ds=(1+S55).*(1+S66)-S56.*S65;
LR=Zref*Ds./(2*S56);
C1=((1-S55).*(1+S66)-S56.*(2-S56))./(Zref.*Ds);
C2=((1+S55).*(1-S66)-S56.*(2-S56))./(Zref.*Ds);

%  L_VBUS=imag(LR)./omega*1e9
L_VBUS_debug=imag(LR)./omega*1e9;

%figure
%plot(f_vbus,L_VBUS_debug)

C_1=omega\imag(C1)*1e9;
C_2=omega\imag(C2)*1e9;
L_VBUS=omega\imag(LR)*1e9;
C_VBUS=min(C_1,C_2);

    % Report Vbus inductance
    tableData{4,1}=[num2str(round(L_VBUS*10)/10),' nH'];
    tableData{4,2}='<= 900 nH';
    if round(L_VBUS*10)/10 <= 900
        tableData{4,3}='Pass';
    else
        tableData{4,3}='Fail';
    end

% Extract mutual inductances
%--------------------------------------------------------------------------
M_D_plus=M_VBUStoLS(f,S,5,6,1,2,Zref,flower,fupper);
M_D_minus=M_VBUStoLS(f,S,5,6,3,4,Zref,flower,fupper);
M_CC=M_VBUStoLS(f,S,5,6,7,8,Zref,flower,fupper);  

    % Report mutual between Vbus and D+, D-
    tableData{5,1}=[num2str(round(M_D_plus*100)/100),'/',num2str(round(M_D_minus*100)/100), ' nH'];
    tableData{5,2}=['<=300 nH'];
    if round(M_D_plus*100)/100<=330 & round(M_D_minus*100)/100<=330
        tableData{5,3}='Pass';
    else
        tableData{5,3}='Fail';
    end
    
    % Report Mutual between Vbus and CC
    tableData{6,1}=[num2str(round(M_CC*100)/100), ' nH'];
    tableData{6,2}=['<=350 nH'];
    if round(M_CC*100)/100 <= 350
        tableData{6,3}='Pass';
    else
        tableData{6,3}='Fail';
    end
   

% Add decaps on the fixtures on both ends
%--------------------------------------------------------------------------
% Vbus bulk caps
C1=inp.Cbulk1;
C2=inp.Cbulk2;

% 2-port cap S-parameters
s11_l=-Zref*sqrt(-1)*2*pi*f*C1./(2+Zref*sqrt(-1)*2*pi*f*C1);
s12_l=2./(2+Zref*sqrt(-1)*2*pi*f*C1);
s11_r=-Zref*sqrt(-1)*2*pi*f*C2./(2+Zref*sqrt(-1)*2*pi*f*C2);
s12_r=2./(2+Zref*sqrt(-1)*2*pi*f*C2);

% Expand to 8-port
Sl=zeros(8,8,length(f));
Sr=zeros(8,8,length(f));
for i=1:2:7
    Sl(i,i+1,:)=1;
    Sl(i+1,i,:)=1;
    Sr(i,i+1,:)=1;
    Sr(i+1,i,:)=1;
end

Sl(5,5,:)=s11_l; Sl(6,6,:)=s11_l; Sl(5,6,:)=s12_l; Sl(6,5,:)=s12_l;
Sr(5,5,:)=s11_r; Sr(6,6,:)=s11_r; Sr(5,6,:)=s12_r; Sr(6,5,:)=s12_r;

% Cascading
idx=[1 3 5 7 2 4 6 8];
Sa=Sl(idx,idx,:); 
Sb=S(idx,idx,:);
Sa=cascade(Sa,Sb);
Sb=Sr(idx,idx,:);
Sc=cascade(Sa,Sb);
idx=[1 5 2 6 3 7 4 8];
Sc=Sc(idx,idx,:);
S=Sc;

spar.f=f; spar.S=S;
setappdata(hMainGui,'spar',spar);

% Resmaple
for ii=1:size(S,1)
    for jj=1:size(S,2)
        tmp=squeeze(S(ii,jj,:));
        S1(ii,jj,:)=interp1(f,tmp,inp.freq_low);
    end
end
S=S1;
    
% Frequency corresponding to S
f=inp.freq_low;


% Report coupling or crosstalk terms
%-------------------------------------------------------------------------
% NEXT/FEXT between D+/D- and VBUS
vbus_usb2(:,1) = squeeze(S(1,5,:) - S(3,5,:))/sqrt(2);
vbus_usb2(:,2) = squeeze(S(1,6,:) - S(3,6,:))/sqrt(2);

    % Pass/fail for VBUS to USB2 coupling
    interp_VBUS_USB2_spec=interSpec(inp.f_VBUS_USB2_COUPLING,inp.VBUS_USB2_COUPLING,f);
    
    % Calculate margin   
    for i=1:size(vbus_usb2,2)
        delta_VBUS_USB2(:,i)= interp_VBUS_USB2_spec(:) - db(vbus_usb2(:,i));
    end
    
    % Report to table
    tableData{3,1}='';
    if min(delta_VBUS_USB2(:))<0
        tableData{3,3}=['Fail'];
    else
        tableData{3,3}=['Pass'];
    end
    tableData{3,2}=[num2str(round(min(delta_VBUS_USB2(:))*10)/10),'dB'];

% NEXT/FEXT between D+/D- and CC
cc_usb2(:,1)=squeeze(S(1,7,:) - S(3,7,:))/sqrt(2);
cc_usb2(:,2)=squeeze(S(1,8,:) - S(3,8,:))/sqrt(2);

%cc1_usb2(:,1) = squeeze(S1(1,7,:) - S1(3,7,:))/sqrt(2);
%cc1_usb2(:,2) = squeeze(S1(1,8,:) - S1(3,8,:))/sqrt(2);
%figure
%semilogx(f,db(cc_usb2(:,1)),f,db(cc1_usb2(:,1)))

    % Pass/fail for CC to USB2 coupling
    interp_CC_USB2_spec=interSpec(inp.f_CC_USB2_COUPLING,inp.CC_USB2_COUPLING,f);
        
    % Calculate margin   
    for i=1:size(cc_usb2,2)
        delta_CC_USB2(:,i)= interp_CC_USB2_spec(:) - db(cc_usb2(:,i));
    end
      
    % Report to table
    tableData{1,1}='';
    if min(delta_CC_USB2(:))<0
       tableData{1,3}=['Fail'];      
    else
       tableData{1,3}=['Pass'];
    end
    tableData{1,2}=[num2str(round(min(delta_CC_USB2(:))*10)/10),'dB'];    
    
% D- and CC (NEXT/FEXT)
cc_usbDminus(:,1) = squeeze(S(3,7,:));
cc_usbDminus(:,2) = squeeze(S(3,8,:)); 
    
    % Pass/fail for CC to USB2 D- (full featured cable) coupling  
    interp_CC_usbDminus_charge_spec=interSpec(inp.f_cc_usbDminus_charge,inp.cc_usbDminus_charge_coupling,f);    
        
    % Calculate margin
    for i=1:size(cc_usbDminus,2)
        delta_CC_usbDminus_charge(:,i)= interp_CC_usbDminus_charge_spec(:) - db(cc_usbDminus(:,i));
    end
    
    % Report to table
    tableData{2,1}='';
    if min(delta_CC_usbDminus_charge(:))<0
        tableData{2,3}=['Fail'];      
    else
        tableData{2,3}=['Pass'];
    end
    tableData{2,2}=[num2str(round(min(delta_CC_usbDminus_charge(:))*10)/10),'dB'];   
    
% Save to workspace 
setappdata(hMainGui,'cc_usb2',cc_usb2);
setappdata(hMainGui,'cc_usbDminus',cc_usbDminus);
setappdata(hMainGui,'vbus_usb2',vbus_usb2);
setappdata(hMainGui,'L_VBUS',L_VBUS);
setappdata(hMainGui,'f_vbus',f_vbus);

% Display to table
setappdata(hMainGui,'tableData_FD',tableData);
set(handles.uitable_result,'Data',tableData);    
popupmenu_Spec_Callback(handles.popupmenu_Spec, '', handles);

% Check and group failed items
k=0;
for i=1:6
    if strcmp(tableData{i,3},'Fail') & i~= 3
       failedItems(k+1)=i;
       k=k+1;
    end
end
if k == 0
    failedItems=[];
end
setappdata(hMainGui,'failedItems',failedItems);   

