% Get LS TD input data
load 'LSTD_inputData.mat';
setappdata(hMainGui,'LSTD',LSTD); 

h = waitbar(0,'Convert to Time-Domain Coupling. Please wait...');

%  Process low speed data to get time-domain   

% Populate ploting popupmenu
itemStr{1}='CC to USB D+/D-';
itemStr{2}='CC to SBU_A/SBU_B';
itemStr{3}='USB D- to CC (BMC Active)';
itemStr{4}='USB D- to CC (BMC Idle)';
itemStr{5}='USB D+/D- to CC (BMC Active)';
itemStr{6}='USB D+/D- to CC (BMC Idle)';
itemStr{7}='USB D+/D- to SBU_A/SBU_B';
itemStr{8}='SBU_A/SBU_B to CC (BMC Active)';
itemStr{9}='SBU_A/SBU_B to CC (BMC Idle)';
itemStr{10}='SBU_A/SBU_B to USB D+/D-';
itemStr{11}='SBU_A to SBU_B';    
set(handles.popupmenu_Spec,'String',itemStr);
set(handles.popupmenu_Spec,'Value',1);

% Set table col and row names
columnname = {'Value', 'Criteria', 'Pass/Fail'};        
rowname = {'CC to D+/D-','CC to SBU',...
           'D- to CC (Active)', 'D- to CC (Idle)',...  
           'D+/D- to CC (Active)', 'D+/D- to CC (Idle)', 'D+/D- to SBU',...  
           'SBU to CC (Active)', 'SBU to CC (Idle)','SBU to D+/D-', 'SBU_A to SBU_B'};
set(handles.uitable_result,'ColumnName', columnname);
set(handles.uitable_result,'RowName', rowname);
set(handles.uitable_result,'Data', []);

%--------------------------------------------------------------------------
% Ref impedance for LS is always 50 ohms
Zref=inp.Z0_low;
sp.Z0=Zref;

% Get the assembled S-parameters with bulkcaps added
spar=getappdata(hMainGui,'spar');
f=spar.f; S=spar.S;

% Add DC
for i=1:size(S,1)
    for j=1:size(S,2)
        S0(i,j,1)=0;
        for k=1:size(S,3)
            S0(i,j,k+1)=S(i,j,k);
        end
    end
end
for i=1:2:size(S,1)-1
    j=i+1;
    S0(i,j,1)=S0(i,j,2);
    S0(j,i,1)=S0(j,i,2);
end
f0=[1;f];
f=f0; S=S0;


fl=LSTD.fl;
fh=LSTD.fh;
npts=LSTD.npts;  
ID=find(f<=fh & f>=fl);
f=f(ID);
S=S(:,:,ID);

% Resample
S0=[];
df1=f(2)-f(1); 
df2=f(11)-f(10); 
fstep=(f(end)-f(1))/(npts-1);

if df1 ~= df2 | df1 > fstep
    freq=[f(1):fstep:f(end)]';
    for ii=1:size(S,1)
        for jj=1:size(S,2)
            tmp=squeeze(S(ii,jj,:));
            S0(ii,jj,:)=interp1(f,tmp,freq);
        end
    end
else
    freq=f;
    S0=S;
end

sp.f=freq; S=S0; 
    
%--------------------------------------------------------------------------

% max time = UI*max_time_mult
max_time_mult=LSTD.max_time_mult;

% Calculate CC2DPDN
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.CC;
tx.Tr=LSTD.Tr.CC;
tx.dataRate=LSTD.dataRate.CC; 
UI=1/tx.dataRate;
max_time=UI*max_time_mult;

portReduction=LSTD.CC2DPDN.portReduction;
sp.S=S(portReduction,portReduction,:);
aggrPorts=LSTD.CC2DPDN.aggrPorts;
vicPorts=LSTD.CC2DPDN.vicPorts; 
aggVic=LSTD.CC2DPDN.aggVic;
respCC2DPDN=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

    % Report CC2DPDN
    p2pMax=respCC2DPDN.p2pMax;
    limit=LSTD.CC2DPDN.limit;
    tableData{1,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{1,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{1,3}='Pass';
    else
        tableData{1,3}='Fail';
    end    

% Save to workspace    
setappdata(hMainGui,'respCC2DPDN',respCC2DPDN);
disp('Done with CC2DPDN')

% Calculate CC2SBU
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.CC;
tx.Tr=LSTD.Tr.CC;
tx.dataRate=LSTD.dataRate.CC; 
UI=1/tx.dataRate;
max_time=UI*max_time_mult;

portReduction=LSTD.CC2SBU.portReduction;
sp.S=S(portReduction,portReduction,:);
aggrPorts=LSTD.CC2SBU.aggrPorts;
vicPorts=LSTD.CC2SBU.vicPorts; 
aggVic=LSTD.CC2SBU.aggVic;
respCC2SBU=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

    % Report CC2SBU    
    %p2pMax=max(max(respCC2SBU.p2pCoupling));
    p2pMax=respCC2SBU.p2pMax;
    limit=LSTD.CC2SBU.limit;
    tableData{2,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{2,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{2,3}='Pass';
    else
        tableData{2,3}='Fail';
    end

% Save to workspace    
setappdata(hMainGui,'respCC2SBU',respCC2SBU);   
disp('Done with CC2SBU')

% Calculate DN2CC
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.DN;
tx.Tr=LSTD.Tr.DN;
tx.dataRate=LSTD.dataRate.DN; 
UI=1/tx.dataRate;
max_time=UI*max_time_mult;

portReduction=LSTD.DN2CC.portReduction;
sp.S=S(portReduction,portReduction,:);
aggrPorts=LSTD.DN2CC.aggrPorts;
vicPorts=LSTD.DN2CC.vicPorts; 
aggVic=LSTD.DN2CC_Act.aggVic;
respDN2CC_Act=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

    % Report DN2CC (BMC Active)
    %p2pMax=max(max(respDN2CC_Act.p2pCoupling));
    p2pMax=respDN2CC_Act.p2pMax;
    limit=LSTD.DN2CC_FF_Active.limit;
    tableData{3,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{3,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{3,3}='Pass';
    else
        tableData{3,3}='Fail';
    end    

aggVic=LSTD.DN2CC_Idl.aggVic;
respDN2CC_Idl=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1);    

    % Report DN2CC (BMC Idle)
    %p2pMax=max(max(respDN2CC_Idl.p2pCoupling));
    p2pMax=respDN2CC_Idl.p2pMax;
    limit=LSTD.DN2CC_FF_Idle.limit;
    tableData{4,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{4,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{4,3}='Pass';
    else
        tableData{4,3}='Fail';
    end   
    
% Save to workspace    
setappdata(hMainGui,'respDN2CC_Act',respDN2CC_Act);  
setappdata(hMainGui,'respDN2CC_Idl',respDN2CC_Idl);  
disp('Done with DN2CC')


% Calculate DPDN2CC
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.DPDN;
tx.Tr=LSTD.Tr.DPDN;
tx.dataRate=LSTD.dataRate.DPDN; 
UI=1/tx.dataRate;
max_time=UI*max_time_mult;
    
portReduction=LSTD.DPDN2CC.portReduction;
sp.S=S(portReduction,portReduction,:);
aggrPorts=LSTD.DPDN2CC.aggrPorts;
vicPorts=LSTD.DPDN2CC.vicPorts; 

aggVic=LSTD.DPDN2CC_Act.aggVic;
respDPDN2CC_Act=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

    % Report DPDN2CC (BMC Active)
    %p2pMax=max(max(respDPDN2CC_Act.p2pCoupling));
    p2pMax=respDPDN2CC_Act.p2pMax;
    limit=LSTD.DPDN2CC_Active.limit;
    tableData{5,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{5,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{5,3}='Pass';
    else
        tableData{5,3}='Fail';
    end        

aggVic=LSTD.DPDN2CC_Idl.aggVic;
respDPDN2CC_Idl=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 
    
    % Report DPDN2CC (BMC Idle)
    %p2pMax=max(max(respDPDN2CC_Idl.p2pCoupling));
    p2pMax=respDPDN2CC_Idl.p2pMax;
    limit=LSTD.DPDN2CC_Idle.limit;
    tableData{6,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{6,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{6,3}='Pass';
    else
        tableData{6,3}='Fail';
    end
    
% Save to workspace    
setappdata(hMainGui,'respDPDN2CC_Act',respDPDN2CC_Act);
setappdata(hMainGui,'respDPDN2CC_Idl',respDPDN2CC_Idl);
disp('Done with DPDN2CC')

%time=respDPDN2CC_Act.t;
%wave=respDPDN2CC_Act.wave;
%for i=1:size(wave,1)
%    for j=1:size(wave,2)
%        i=1; j=1;
%        figure
%        plot(time,squeeze(wave(i,j,:)))
%        p2p=2*max(abs(squeeze(wave(i,j,:))))
%    end
%end


% Calculate DPDN2SBU
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.DPDN;
tx.Tr=LSTD.Tr.DPDN;
tx.dataRate=LSTD.dataRate.DPDN;
tx.UI=1/tx.dataRate;
%max_time=tx.Tr*10;
max_time=tx.UI*max_time_mult;
    
portReduction=LSTD.DPDN2SBU.portReduction;
sp.S=S(portReduction,portReduction,:);
aggrPorts=LSTD.DPDN2SBU.aggrPorts;
vicPorts=LSTD.DPDN2SBU.vicPorts; 
aggVic=LSTD.DPDN2SBU.aggVic;
respDPDN2SBU=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

    % Report DPDN2SBU    
    %p2pMax=max(max(respDPDN2SBU.p2pCoupling));
    p2pMax=respDPDN2SBU.p2pMax;
    limit=LSTD.DPDN2SBU.limit;
    tableData{7,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{7,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{7,3}='Pass';
    else
        tableData{7,3}='Fail';
    end
 
% Save to workspace    
setappdata(hMainGui,'respDPDN2SBU',respDPDN2SBU);   
disp('Done with DPDN2SBU')

%time=respDPDN2SBU.t;
%wave=respDPDN2SBU.wave;
%for i=1:size(wave,1)
%    for j=1:size(wave,2)
%        i=1; j=1;
%        figure
%        plot(time,squeeze(wave(i,j,:)))
%        p2p=2*max(abs(squeeze(wave(i,j,:))))
%    end
%end

% Calculate SBU2CC
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.SBU;
tx.Tr=LSTD.Tr.SBU;
tx.dataRate=LSTD.dataRate.SBU; 
UI=1/tx.dataRate;
max_time=UI*max_time_mult;
    
portReduction=LSTD.SBU2CC.portReduction;
sp.S=S(portReduction,portReduction,:);
aggrPorts=LSTD.SBU2CC.aggrPorts;
vicPorts=LSTD.SBU2CC.vicPorts; 
aggVic=LSTD.SBU2CC_Act.aggVic;
respSBU2CC_Act=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

    % Report SBU2CC (BMC Active)
    %p2pCp=respSBU2CC_Act.p2pCoupling;
    p2pMax=respSBU2CC_Act.p2pMax;
    %cpSum3p56agg1=p2pCp(1,1)+max(p2pCp(3:4,1));
    %cpSum4p56agg1=p2pCp(1,2)+max(p2pCp(3:4,2));
    %cpSum3p56agg2=p2pCp(2,1)+max(p2pCp(3:4,1));
    %cpSum4p56agg2=p2pCp(2,2)+max(p2pCp(3:4,2));
    %p2pMax=max([cpSum3p56agg1 cpSum4p56agg1 cpSum3p56agg2 cpSum4p56agg2]);
    %p2pMax=max(max(respSBU2CC_Act.p2pCoupling));
    limit=LSTD.SBU2CC_Active.limit;
    tableData{8,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{8,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{8,3}='Pass';
    else
        tableData{8,3}='Fail';
    end    
    
aggVic=LSTD.SBU2CC_Idl.aggVic;
respSBU2CC_Idl=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 
    
    % Report SBU2CC (BMC Idle)
    %p2pCp=respSBU2CC_Idl.p2pCoupling;
    p2pMax=respSBU2CC_Idl.p2pMax;
    %cpSum3p56agg1=p2pCp(1,1)+max(p2pCp(3:4,1));
    %cpSum4p56agg1=p2pCp(1,2)+max(p2pCp(3:4,2));
    %cpSum3p56agg2=p2pCp(2,1)+max(p2pCp(3:4,1));
    %cpSum4p56agg2=p2pCp(2,2)+max(p2pCp(3:4,2));
    %p2pMax=max([cpSum3p56agg1 cpSum4p56agg1 cpSum3p56agg2 cpSum4p56agg2]);
    %p2pMax=max(max(respSBU2CC_Idl.p2pCoupling));
    limit=LSTD.SBU2CC_Idle.limit;
    tableData{9,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{9,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{9,3}='Pass';
    else
        tableData{9,3}='Fail';
    end

% Save to workspace    
setappdata(hMainGui,'respSBU2CC_Act',respSBU2CC_Act); 
setappdata(hMainGui,'respSBU2CC_Idl',respSBU2CC_Idl); 
disp('Done with SBU2CC')

 
% Calculate SBU2DPDN
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.SBU;
tx.Tr=LSTD.Tr.SBU;
tx.dataRate=LSTD.dataRate.SBU; 
UI=1/tx.dataRate;
max_time=UI*max_time_mult;
    
portReduction=LSTD.SBU2DPDN.portReduction;
sp.S=S(portReduction,portReduction,:);    
aggrPorts=LSTD.SBU2DPDN.aggrPorts;
vicPorts=LSTD.SBU2DPDN.vicPorts; 
aggVic=LSTD.SBU2DPDN.aggVic;    
respSBU2DPDN=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

    % Report SBU2DPDN
    %p2pCp=respSBU2DPDN.p2pCoupling;
    p2pMax=respSBU2DPDN.p2pMax;
    %cpSum3p56agg1=p2pCp(1,1)+max(p2pCp(3:4,1));
    %cpSum4p56agg1=p2pCp(1,2)+max(p2pCp(3:4,2));
    %cpSum3p56agg2=p2pCp(2,1)+max(p2pCp(3:4,1));
    %cpSum4p56agg2=p2pCp(2,2)+max(p2pCp(3:4,2));
    %p2pMax=max([cpSum3p56agg1 cpSum4p56agg1 cpSum3p56agg2 cpSum4p56agg2]);
    %p2pMax=max(max(respSBU2DPDN.p2pCoupling));
    limit=LSTD.SBU2DPDN.limit;
    tableData{10,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{10,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{10,3}='Pass';
    else
        tableData{10,3}='Fail';
    end

%figure
%plot(respSBU2DPDN.t,squeeze(respSBU2DPDN.wave(1,1,:)),...     
%     respSBU2DPDN.t,squeeze(respSBU2DPDN.wave(4,1,:)));

%figure
%plot(respSBU2DPDN.t,squeeze(respSBU2DPDN.wave(1,2,:)),...    
%     respSBU2DPDN.t,squeeze(respSBU2DPDN.wave(4,2,:))); 



% Save to workspace
setappdata(hMainGui,'respSBU2DPDN',respSBU2DPDN); 
disp('Done with SBU2DPDN')

% Calculate SBUA2SBUB
%--------------------------------------------------------
tx.Vsw=LSTD.Vsw.SBU;
tx.Tr=LSTD.Tr.SBU;
tx.dataRate=LSTD.dataRate.SBU; 
UI=1/tx.dataRate;
max_time=UI*max_time_mult;

portReduction=LSTD.SBU2SBU.portReduction;
sp.S=S(portReduction,portReduction,:);    
aggrPorts=LSTD.SBU2SBU.aggrPorts;
vicPorts=LSTD.SBU2SBU.vicPorts; 
aggVic=LSTD.SBU2SBU.aggVic;    
respSBU2SBU=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

    % Report SBU2SBU
    %p2pMax=max(max(respSBU2SBU.p2pCoupling));
    p2pMax=respSBU2SBU.p2pMax;
    limit=LSTD.SBU2SBU.limit;
    tableData{11,1}=[num2str(round(p2pMax*10)/10),' mV'];
    tableData{11,2}=['<= ', num2str(limit), ' mV'];
    if round(p2pMax*10)/10 <= limit
        tableData{11,3}='Pass';
    else
        tableData{11,3}='Fail';
    end

% Save to workspace
setappdata(hMainGui,'respSBU2SBU',respSBU2SBU);  
disp('Done with SBU2SBU')

% Display to table
setappdata(hMainGui,'tableData_TD_Full',tableData);
set(handles.uitable_result,'Data',tableData);    
popupmenu_Spec_Callback(handles.popupmenu_Spec, '', handles);
disp('Done with Converting to Time Domain Coupling')


close(h)
