% Get LS TD input data
load 'LSTD_inputData.mat';
setappdata(hMainGui,'LSTD',LSTD); 
max_time_mult=LSTD.max_time_mult;
    
h = waitbar(0,'Convert to Time-Domain Coupling. Please wait...');

%  Process low speed data to get time-domain   

% Get failed items in FD
failedItems=getappdata(hMainGui,'failedItems');   

if isempty(failedItems)
    disp('No failed frequency domain items');
else
    % Collect the fialed items
    itemTokeep=[]; caseNum=[];
    for i=1:length(failedItems)
        switch failedItems(i)
            case 1
                itemTokeep=[itemTokeep 1 5 6];
                caseNum=[caseNum 1 4];
            case 2
                itemTokeep=[itemTokeep 3 4];
                caseNum=[caseNum 3];
            case 4
                itemTokeep=[itemTokeep 2 8 9];
                caseNum=[caseNum 2 6];
            case 5
                itemTokeep=[itemTokeep 7 10];
                caseNum=[caseNum 5 7];
            case 6
                itemTokeep=[itemTokeep 11];
                caseNum=[caseNum 8];
        end
    end
    itemTokeep=sort(itemTokeep);
    caseNum=sort(caseNum);
    setappdata(hMainGui,'itemTokeep',itemTokeep);
    
    % Full popup menu
    itemStr{1}='CC to USB D+/D-';
    itemStr{2}='CC to SBU_A/SBU_B';
    itemStr{3}='USB D- to CC (BMC Active)';
    itemStr{4}='USB D- to CC (BMC Idle)';
    itemStr{5}='USB D+/D- to CC (BMC Active)';
    itemStr{6}='USB D+/D- to CC (BMC Idle)';
    itemStr{7}='USB D+/D- to SBU_A/SBU_B';
    itemStr{8}='SBU_A/SBU_B to CC (BMC Active)';
    itemStr{9}='SBU_A/SBU_B to CC (BMC Idle)';
    itemStr{10}='SBU_A/SBU_B to USB D+/D-';
    itemStr{11}='SBU_A to SBU_B';    
    
    % Filter out unwanted items
    for i=1:length(itemTokeep)
        Str{i}=itemStr{itemTokeep(i)};
    end
    setappdata(hMainGui,'StrForPartialTD',Str);
    
    set(handles.popupmenu_Spec,'String',Str);
    set(handles.popupmenu_Spec,'Value',1);

    % Set table col and row names
    columnname = {'Value', 'Criteria', 'Pass/Fail'};        
    rowname = Str;
    set(handles.uitable_result,'ColumnName', columnname);
    set(handles.uitable_result,'RowName', rowname);
    set(handles.uitable_result,'Data', []);

    %--------------------------------------------------------------------------
    % Ref impedance for LS is always 50 ohms
    Zref=inp.Z0_low;
    sp.Z0=Zref;

    % Get the assembled S-parameters with bulkcaps added
    spar=getappdata(hMainGui,'spar');
    f=spar.f; S=spar.S;

    % Add DC
    for i=1:size(S,1)
        for j=1:size(S,2)
            S0(i,j,1)=0;
            for k=1:size(S,3)
                S0(i,j,k+1)=S(i,j,k);
            end
        end
    end
    for i=1:2:size(S,1)-1
        j=i+1;
        S0(i,j,1)=S0(i,j,2);
        S0(j,i,1)=S0(j,i,2);
    end
    f0=[1;f];

    f=f0; S=S0;

    % Resample
    fl=LSTD.fl;
    fh=LSTD.fh;
    npts=LSTD.npts;
    ID=find(f<=fh & f>=fl);
    f=f(ID);
    S=S(:,:,ID);

    S0=[];
    df1=f(2)-f(1); df2=f(11)-f(10); %npts=length(f);
    fstep=(f(end)-f(1))/(npts-1);
    %if fstep > 200e3
    %    fstep=200e3;
    %end

    if df1 ~= df2 | df1 > fstep
        freq=[f(1):fstep:f(end)]';
        for ii=1:size(S,1)
            for jj=1:size(S,2)
                tmp=squeeze(S(ii,jj,:));
                S0(ii,jj,:)=interp1(f,tmp,freq);
            end
        end
    else
        freq=f;
        S0=S;
    end
       

    sp.f=freq; S=S0;

    %--------------------------------------------------------------------------
    
    k=1;
    for ii=1:length(caseNum)
        switch caseNum(ii)
            case 1    % Calculate CC2DPDN
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.CC;
                tx.Tr=LSTD.Tr.CC;
                tx.dataRate=LSTD.dataRate.CC; 
                UI=1/tx.dataRate;
                max_time=UI*max_time_mult;

                portReduction=LSTD.CC2DPDN.portReduction;
                sp.S=S(portReduction,portReduction,:);
                aggrPorts=LSTD.CC2DPDN.aggrPorts;
                vicPorts=LSTD.CC2DPDN.vicPorts; 
                aggVic=LSTD.CC2DPDN.aggVic;
                respCC2DPDN=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

                    % Report CC2DPDN                    
                    p2pMax=respCC2DPDN.p2pMax;
                    limit=LSTD.CC2DPDN.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                % Save to workspace    
                setappdata(hMainGui,'respCC2DPDN',respCC2DPDN);
                disp('Done with CC2DPDN')
            case 2    % Calculate CC2SBU
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.CC;
                tx.Tr=LSTD.Tr.CC;
                tx.dataRate=LSTD.dataRate.CC; 
                UI=1/tx.dataRate;
                max_time=UI*max_time_mult;

                portReduction=LSTD.CC2SBU.portReduction;
                sp.S=S(portReduction,portReduction,:);
                aggrPorts=LSTD.CC2SBU.aggrPorts;
                vicPorts=LSTD.CC2SBU.vicPorts; 
                aggVic=LSTD.CC2SBU.aggVic;
                respCC2SBU=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

                    % Report CC2SBU    
                    p2pMax=respCC2SBU.p2pMax;
                    limit=LSTD.CC2SBU.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                % Save to workspace    
                setappdata(hMainGui,'respCC2SBU',respCC2SBU);   
                disp('Done with CC2SBU')
            case 3    % Calculate DN2CC
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.DN;
                tx.Tr=LSTD.Tr.DN;
                tx.dataRate=LSTD.dataRate.DN; 
                UI=1/tx.dataRate;
                max_time=UI*max_time_mult;

                portReduction=LSTD.DN2CC.portReduction;
                sp.S=S(portReduction,portReduction,:);
                aggrPorts=LSTD.DN2CC.aggrPorts;
                vicPorts=LSTD.DN2CC.vicPorts; 
                aggVic=LSTD.DN2CC_Act.aggVic;
                respDN2CC_Act=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

                    % Report DN2CC (BMC Active)                    
                    p2pMax=respDN2CC_Act.p2pMax;
                    limit=LSTD.DN2CC_FF_Active.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                aggVic=LSTD.DN2CC_Idl.aggVic;
                respDN2CC_Idl=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1);    

                    % Report DN2CC (BMC Idle)
                    p2pMax=respDN2CC_Idl.p2pMax;
                    limit=LSTD.DN2CC_FF_Idle.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                % Save to workspace    
                setappdata(hMainGui,'respDN2CC_Act',respDN2CC_Act);  
                setappdata(hMainGui,'respDN2CC_Idl',respDN2CC_Idl);  
                disp('Done with DN2CC')
            case 4    % Calculate DPDN2CC
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.DPDN;
                tx.Tr=LSTD.Tr.DPDN;
                tx.dataRate=LSTD.dataRate.DPDN; 
                UI=1/tx.dataRate;
                max_time=UI*max_time_mult;

                portReduction=LSTD.DPDN2CC.portReduction;
                sp.S=S(portReduction,portReduction,:);
                aggrPorts=LSTD.DPDN2CC.aggrPorts;
                vicPorts=LSTD.DPDN2CC.vicPorts; 

                aggVic=LSTD.DPDN2CC_Act.aggVic;
                respDPDN2CC_Act=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

                    % Report DPDN2CC (BMC Active)
                    p2pMax=respDPDN2CC_Act.p2pMax;
                    limit=LSTD.DPDN2CC_Active.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end        
                    k=k+1;

                aggVic=LSTD.DPDN2CC_Idl.aggVic;
                respDPDN2CC_Idl=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

                    % Report DPDN2CC (BMC Idle)
                    p2pMax=respDPDN2CC_Idl.p2pMax;
                    limit=LSTD.DPDN2CC_Idle.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                % Save to workspace    
                setappdata(hMainGui,'respDPDN2CC_Act',respDPDN2CC_Act);
                setappdata(hMainGui,'respDPDN2CC_Idl',respDPDN2CC_Idl);
                disp('Done with DPDN2CC') 
            case 5   % Calculate DPDN2SBU
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.DPDN;
                tx.Tr=LSTD.Tr.DPDN;
                tx.dataRate=LSTD.dataRate.DPDN;
                tx.UI=1/tx.dataRate;                
                max_time=tx.UI*max_time_mult;

                portReduction=LSTD.DPDN2SBU.portReduction;
                sp.S=S(portReduction,portReduction,:);
                aggrPorts=LSTD.DPDN2SBU.aggrPorts;
                vicPorts=LSTD.DPDN2SBU.vicPorts; 
                aggVic=LSTD.DPDN2SBU.aggVic;
                respDPDN2SBU=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

                    % Report DPDN2SBU    
                    p2pMax=respDPDN2SBU.p2pMax;
                    limit=LSTD.DPDN2SBU.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                % Save to workspace    
                setappdata(hMainGui,'respDPDN2SBU',respDPDN2SBU);   
                disp('Done with DPDN2SBU')

            case 6   % Calculate SBU2CC
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.SBU;
                tx.Tr=LSTD.Tr.SBU;
                tx.dataRate=LSTD.dataRate.SBU; 
                UI=1/tx.dataRate;
                max_time=UI*max_time_mult;

                portReduction=LSTD.SBU2CC.portReduction;
                sp.S=S(portReduction,portReduction,:);
                aggrPorts=LSTD.SBU2CC.aggrPorts;
                vicPorts=LSTD.SBU2CC.vicPorts; 
                aggVic=LSTD.SBU2CC_Act.aggVic;
                respSBU2CC_Act=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

                    % Report SBU2CC (BMC Active)                    
                    p2pMax=respSBU2CC_Act.p2pMax;
                    limit=LSTD.SBU2CC_Active.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                aggVic=LSTD.SBU2CC_Idl.aggVic;
                respSBU2CC_Idl=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,1); 

                    % Report SBU2CC (BMC Idle)
                    p2pMax=respSBU2CC_Idl.p2pMax;
                    limit=LSTD.SBU2CC_Idle.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                % Save to workspace    
                setappdata(hMainGui,'respSBU2CC_Act',respSBU2CC_Act); 
                setappdata(hMainGui,'respSBU2CC_Idl',respSBU2CC_Idl); 
                disp('Done with SBU2CC')
            case 7   % Calculate SBU2DPDN
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.SBU;
                tx.Tr=LSTD.Tr.SBU;
                tx.dataRate=LSTD.dataRate.SBU; 
                UI=1/tx.dataRate;
                max_time=UI*max_time_mult;

                portReduction=LSTD.SBU2DPDN.portReduction;
                sp.S=S(portReduction,portReduction,:);    
                aggrPorts=LSTD.SBU2DPDN.aggrPorts;
                vicPorts=LSTD.SBU2DPDN.vicPorts; 
                aggVic=LSTD.SBU2DPDN.aggVic;    
                respSBU2DPDN=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

                    % Report SBU2DPDN
                    p2pMax=respSBU2DPDN.p2pMax;
                    limit=LSTD.SBU2DPDN.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end
                    k=k+1;

                % Save to workspace
                setappdata(hMainGui,'respSBU2DPDN',respSBU2DPDN); 
                disp('Done with SBU2DPDN')
            case 8   % Calculate SBUA2SBUB
                %--------------------------------------------------------
                tx.Vsw=LSTD.Vsw.SBU;
                tx.Tr=LSTD.Tr.SBU;
                tx.dataRate=LSTD.dataRate.SBU; 
                UI=1/tx.dataRate;
                max_time=UI*max_time_mult;

                portReduction=LSTD.SBU2SBU.portReduction;
                sp.S=S(portReduction,portReduction,:);    
                aggrPorts=LSTD.SBU2SBU.aggrPorts;
                vicPorts=LSTD.SBU2SBU.vicPorts; 
                aggVic=LSTD.SBU2SBU.aggVic;    
                respSBU2SBU=getTDresp(sp,aggrPorts,vicPorts,aggVic,tx,max_time,0); 

                    % Report SBU2SBU
                    p2pMax=respSBU2SBU.p2pMax;
                    limit=LSTD.SBU2SBU.limit;
                    tableData{k,1}=[num2str(round(p2pMax*10)/10),' mV'];
                    tableData{k,2}=['<= ', num2str(limit), ' mV'];
                    if round(p2pMax*10)/10 <= limit
                        tableData{k,3}='Pass';
                    else
                        tableData{k,3}='Fail';
                    end

                % Save to workspace
                setappdata(hMainGui,'respSBU2SBU',respSBU2SBU);  
                disp('Done with SBU2SBU')
        end
    end

    % Display to table
    setappdata(hMainGui,'tableData_TD_Partial',tableData);
    set(handles.uitable_result,'Data',tableData);    
    popupmenu_Spec_Callback(handles.popupmenu_Spec, '', handles);
    disp('Done with Converting to Time Domain Coupling')

end

close(h)

